/***************************************************************************
* Train.h                                                                  *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       08/18/99  arvo    Completely rewritten.                            *
*       08/15/99  arvo    Removed fixed-length arrays.                     *
*       12/21/97  arvo    major cleanup.                                   *
*       12/01/96  guo     Initial codiing.                                 *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __TRAIN_INCLUDED__
#define __TRAIN_INCLUDED__

//#include <iostream.h>
#include <tcl.h>
#include <tk.h>
#include "../rec/Command.h"
#include "../rec/FeatureRecognizer.h"
#include "../rec/BasicGrouper.h"

enum TrainModeType {
    recognition_mode,
    grouping_mode,
    training_mode
    };

enum StartupModeType {
    interactive_mode,
    regression_only,
    thumbnails_only
    };

class Train : public FeatureRecognizer {

  public: // All the public methods.
    Train( const char *base_path = NULL );  // Path to Tcl/Tk script.
   ~Train();
    void Run( 
        int   argc, 
        char *argv[], 
        const char *data_dir, 
        const char *font_dir = NULL, 
        int mode = interactive_mode 
        );

  private: // Static methods (to enable callbacks from Tcl/Tk).
    static int Startup( Tcl_Interp *interp );
    static int SendMessage( ClientData, Tcl_Interp *, int argc, const char *argv[] );
   
  private: // Non-static methods.
    int  Dispatch( int argc, const char *argv[] );
    void Load( const char *dname );
    void WriteTrainingFile( const char *name );
    void RedrawSample( int k ) const;
    void Tcl_Command( const char *, const char * = NULL, const char * = NULL ) const;
    void Tcl_Command( const char *, int      ) const;
    void Tcl_Command( const char *, int, int ) const;
    int  BadArgs( int argc, const char *argv[] );
    void DrawLine( const Vec2 &P, const Vec2 &Q, const char *color = "black" ) const;
    void UpdateProfile( Profile &prof );
    void UpdateStatistics( int choice );
    void RunRecognizer( const Multiline & );
    void RunRegressionTest( const char *frame );
    void ShowThumbnails( const char *frame ) const;
    void DrawThumbnail( int size, const Multiline & ) const;
    void DrawMultiline( const Multiline &, const char *color = "black" ) const;
    void FillMultiline( const Multiline &, const char *color = "black" ) const;
    int  ShowActualChar( const Multiline &strokes, Symbol *symb ) const;

    void DrawMesh( const Multiline &, float t, const char *color = "black" ) const;
    void DrawTriangle( const Pnode *A, const Pnode *B, const Pnode *C, float t, const char *color = "black" ) const;


  private: // Methods that handle corresponding Tcl commands.
    DispatchDecl( move_to       );
    DispatchDecl( draw_to       );
    DispatchDecl( end_line      );
    DispatchDecl( quit          );
    DispatchDecl( post_proc     );
    DispatchDecl( set_mode      );
    DispatchDecl( timed_out     );
    DispatchDecl( yes_button    );
    DispatchDecl( no_button     );
    DispatchDecl( load_data     );
    DispatchDecl( show_profile  );
    DispatchDecl( hide_profile  );
    DispatchDecl( new_sample    );
    DispatchDecl( erase_sample  );
    DispatchDecl( save_samples  );
    DispatchDecl( clear         );
    DispatchDecl( make_exemplar );
    DispatchDecl( print_lines   );
    DispatchDecl( show_stats    );
    DispatchDecl( change_stats  );
    DispatchDecl( run_reg_test  );
    DispatchDecl( next_symbol   );
    DispatchDecl( prev_symbol   );
    DispatchDecl( show_thumb    );
    DispatchDecl( thumb_anim    );
    DispatchDecl( morph         );
    DispatchDecl( group         );

  private: // Non-static data.
    int curr_sample;    // The sample currently being displayed.
    int curr_choice;    // Statistics currently being dispayed for this choice.
    int stats_flag;     // Turns display of statistics on and off.
    int num_samples;    // Number of samples recorded for a new symbol.
    int mode;           // Recognition or Training mode.
    int exemplar_index; // Use this sample as the "exemplar".
    Symbol    *symbol;  // The recognized symbol.
    Multiline Strokes;  // The current set of stokes being input.
    Multiline Samples[ Max_Samples ];
    BasicGrouper grouper;

  private: // Static data (to enable callbacks from Tcl/Tk).
    static Tcl_Interp *interp;               // The Tcl interpreter.
    static char data_dir[ Max_Path_Length ]; // Where the training data is.
    static char font_dir[ Max_Path_Length ]; // Where the font descriptions live.
    static char base_dir[ Max_Path_Length ]; // Where the Tcl/Tk script is.
    static Profile profile;                  // Recognition profile.
    static Train *train;                     // Static pointer to Train instance.
    static int debug;    
    static int do_post_proc;
    static const char *proc_name;
  };


extern const char *FilterTclName( 
    const char *name 
    );

#endif




